Network network = new Network();
HashMap<String, Node> nodeIndex = new HashMap<String, Node>();

float scaleFactor = 1.0f;
float translateX = 0.0f;
float translateY = 0.0f;

Node getNode(String name) {
  if (!nodeIndex.containsKey(name)) {
    nodeIndex.put(name, new Node(name));
  }

  return nodeIndex.get(name);
}

void setup() {
  size(720, 720, FX2D);
  //randomSeed(1002);

  // create network graph
  JSONObject data = loadJSONObject("../data-clean.json");
  String username = data.getString("username");
  String timestamp = data.getString("timestamp");

  JSONObject followers = data.getJSONObject("followers");

  Node root = getNode(username);
  root.isRoot = true;
  root.position = new PVector(width / 2, height / 2);

  for (Object key : followers.keys()) {
    String name = (String)key;
    JSONObject user = followers.getJSONObject(name);
    JSONArray theirfollowers = user.getJSONArray("followers");

    int followersCount = user.getInt("followers_count");
    int followingCount = user.getInt("following_count");

    float metric = followingCount / (float)followersCount;
    if (metric < 0.3) {
      break;
    }

    println("Followers: " + followersCount + " Follows: " + followingCount);

    Node node = getNode(name);
    root.addEdges(node);

    int count = 0;

    for (String followerName : theirfollowers.getStringArray()) {
      Node followerNode = getNode(followerName);
      node.addEdges(followerNode);
      count++;

      if (count > 10) 
        break;
    }
  }

  // add all nodes
  for (String key : nodeIndex.keySet()) {
    network.addNodes(nodeIndex.get(key));
  }

  println("Added " + nodeIndex.keySet().size() + " nodes...");
}

void draw() {
  background(255);

  translate(translateX, translateY);
  scale(scaleFactor);

  if (frameCount < 3) {
    text("iterating...", 200, 200);
  } else if (frameCount == 3) {
    for (int i = 0; i < 3000; i++) {
      network.iterate();
    }
  } else {
    network.render();
  }

  surface.setTitle("Framerate: " + round(frameRate));
}

void mouseWheel(MouseEvent e) {
  scaleFactor += e.getAmount() / 100;
  translateX -= e.getAmount() * mouseX / 100;
  translateY -= e.getAmount() * mouseY / 100;
}


void mouseReleased() {
  scaleFactor = 1.0;
  translateX = 0.0;
  translateY = 0.0;
}
